/* dt_hash.h
**
** data type for generic hash table
** dtlib-lite library
**
** Copyright (C) 2002, 2003, Wayne Marshall
** All rights reserved, worldwide.
**
** wcm, 2002.11.25 - 2003.02.04
** ===
*/
#ifndef  DT_HASH_H
#define  DT_HASH_H 1


struct hash {
    /* opaque internal representation: */
    struct hash_  *hash;
    /* public methods: */
    /* hash table stats: */
    int      (*size)(struct hash *);
    int      (*items)(struct hash *);
    int      (*is_empty)(struct hash *);
    float    (*load)(struct hash *);
    int      (*chains)(struct hash *);
    int      (*collides)(struct hash *);
    /* check the hash value of a key produced by installed hash_func(): */
    unsigned int (*hash_check)(struct hash *, const void *key);
    /* insert/lookup/exists/remove: */
    void    *(*insert)(struct hash *, const void *key, void *data);
    void    *(*lookup)(struct hash *, const void *key);
    int      (*exists)(struct hash *, const void *key);
    void    *(*remove)(struct hash *, const void *key);
    /* extended methods: */
    int      (*retrieve)
                 (
                     struct hash  *myhash,
                     void         *lookup_key,
                     void        **key,
                     void        **data
                 );
    int      (*extract)
                 (
                     struct hash  *myhash,
                     void         *lookup_key,
                     void        **key,
                     void        **data
                 );

    /* apply a "visitor" function to each item in hash: */
    void     (*apply)
                 (
                     struct hash *,
                     void (*visitor)(const void *key, void *data, void *xtra),
                     void *xtra
                 );

    /* apply user-defined item destructor to each item in hash: */
    void     (*clear)
                 (
                     struct hash *,
                     void (*free_item)(void *key, void *data)
                 );

    /* hash supplies constructor for cursor object: */
    struct hash_cursor  *(*new_cursor)(struct hash *);

    /* rehash() to change table size: */
    struct hash *
        (*rehash)
            (
                struct hash   *myhash,
                int            tblsize
            );

    /* destructor: */
    void    (*free)(struct hash *);
};

typedef struct hash  *dt_Hash;


/* hash creator: */
extern
dt_Hash
new_hash
(
    int            tblsize,
    unsigned int (*hash_func)(const void *key),
    int          (*key_compare)(const void *key1, const void *key2)
);



#endif  /* DT_HASH_H */
