/* dt_list.h
**
** data type for generic linked list
** dtlib-lite library 
**
** Copyright (C) 2002, 2003, Wayne Marshall
** All rights reserved, worldwide.
**
** wcm, 2002.11.25 - 2003.02.04
** ===
*/
#ifndef  DT_LIST_H
#define  DT_LIST_H  1



struct list {
    /* opaque internal representation: */
    struct list_  *list;
    /* public methods: */
    /* xpin_list table stats: */
    int      (*items)(struct list *);
    int      (*is_empty)(struct list *);
    void     (*push)(struct list *, void *data);
    void    *(*peek)(struct list *);
    void    *(*pop)(struct list *);
    void     (*addend)(struct list *, void *data);
    void     (*reverse)(struct list *);
    /* search/exists/remove/replace: */
    void    *(*search)
                (
                    struct list *,
                    int (*compare)(const void *key1, const void *key2),
                    const void *key
                );
    int      (*exists)(struct list *, const void *key);
    void    *(*remove)(struct list *, const void *key);
    void    *(*replace)(struct list *, const void *key);
    /* apply a "visitor" function to each item in list: */
    void     (*apply)
                 (
                     struct list *,
                     void (*visitor)(void *data, void *xtra),
                     void *xtra
                 );

    /* apply user-defined item destructor to each item in list: */
    void     (*clear)
                 (
                     struct list *,
                     void (*free_item)(void *data)
                 );

    /* destructor: */
    void    (*free)(struct list *);
};

typedef struct list *dt_List;


/* list creator: */
extern dt_List new_list();


#endif  /* DT_LIST_H */
