/* xpin_handlers.h
**
** XpinDoc
** an XML parse instance document processing utility
**
** Copyright (C) 2002, 2003, Wayne Marshall
** All rights reserved, worldwide.
**
** wcm, 2002.05.20 - 2003.09.30
** ===
*/
#ifndef  XPIN_HANDLERS_H
#define  XPIN_HANDLERS_H

#include "xpindoc.h"   /*typedef for Xpin_Handler function signature*/
#include "dt_hash.h"
#include "dt_list.h"

typedef unsigned char  Xpin_Char;


/* xpin_handler_suite object (contained in XpinDoc object (xpin.c): */

struct xpin_handler_suite {
    /* default handlers: */
    Xpin_Handler    start_document_handler;
    Xpin_Handler    end_document_handler;
    Xpin_Handler    start_element_handler;
    Xpin_Handler    end_element_handler;
    Xpin_Handler    ns_start_element_handler;
    Xpin_Handler    ns_end_element_handler;
    Xpin_Handler    pi_handler;
    Xpin_Handler    start_cdata_handler;
    Xpin_Handler    end_cdata_handler;
    Xpin_Handler    error_handler;
    /* ... */
    /* handler hashes: */
    dt_Hash    start_tag_hash;
    dt_Hash    end_tag_hash;
    dt_Hash    pi_hash;
    /* ... */
    /* handler lists: */
    dt_List    tagpath_list;
    /* ... */
};

/* constructor: */
extern
struct xpin_handler_suite *
new_xpin_handler_suite(void);

/* destructor: */
extern
void
free_xpin_handler_suite(struct xpin_handler_suite *);



/* user method installed in XpinDoc object (xpin.c): */
extern
int
xpin_set_handler
(
    struct xpindoc  *X,
    const char      *keystr,
    Xpin_Handler     handler
);



/* all meta handlers exported for xpin.c here: */
extern
void
xpin_meta_start_document_handler(struct xpindoc *);

extern
void
xpin_meta_end_document_handler(struct xpindoc *);

extern
void
xpin_meta_start_element_handler
(
    struct xpindoc     *X,
    const Xpin_Char    *local_name,
    const Xpin_Char    *ns_uri,
    const Xpin_Char    *ns_prefix,
    const Xpin_Char    *qname,
    int                 attr_count,
    struct xpin_attr  **attributes
);

extern
void
xpin_meta_end_element_handler
(
    struct xpindoc   *X
);

extern
void
xpin_meta_pi_handler
(
    struct xpindoc    *X,
    const Xpin_Char   *pi_name,
    const Xpin_Char   *pi_data
);


extern
void
xpin_meta_start_cdata_handler(struct xpindoc *);


extern
void
xpin_meta_end_cdata_handler(struct xpindoc *);


extern
void
xpin_meta_chardata_handler
(
    struct xpindoc    *X,
    const Xpin_Char   *chardata,
    int                len
);


#endif  /* XPIN_HANDLERS_H */
