/* xpin_pi.c
**
** XpinDoc
** an XML parse instance document processing utility
**
** Copyright (C) 2002, 2003, Wayne Marshall
** All rights reserved, worldwide.
**
** wcm, 2002.05.20 - 2003.09.30
** ===
*/
#include <assert.h>
#include <stdlib.h>

/* our headers: */
#include "xpindoc.h"
#include "xpin_pi.h"


/* PI OBJECTS: */

struct xpin_pi_ {
    const char    *name;
    const char    *data;
};


/* internal prototypes: */

/* PI METHODS: */
static
const char *
pi_get_target(struct xpin_pi *);

static
const char *
pi_get_data(struct xpin_pi *);



/* pi method definitions: */
static
const char *
pi_get_target(struct xpin_pi *P)
{
    return P->pi->name;
}

static
const char *
pi_get_data(struct xpin_pi *P)
{
    return P->pi->data;
}


/* exported (xpin_pi.h): */

/* PI CONSTRUCTOR: */
struct xpin_pi *
new_xpin_pi()
{
    struct xpin_pi   *P = NULL;
    struct xpin_pi_  *pi = NULL;

    P = (struct xpin_pi *)malloc(sizeof(struct xpin_pi));
    if(P == NULL)  return NULL;

    pi = (struct xpin_pi_ *)malloc(sizeof(struct xpin_pi_));
    if(pi == NULL){
        free(P);
        return NULL;
    }

    pi->name  = NULL;
    pi->data  = NULL;

    P->pi = pi;

    P->target = &pi_get_target;
    P->data   = &pi_get_data;

    return P;
}/*end new_xpin_pi()*/



/* RE-USE PI OBJECT: */
struct xpin_pi *
clear_xpin_pi(struct xpin_pi *PI)
{
    struct xpin_pi_  *pi = NULL;

    assert((PI != NULL) && (PI->pi != NULL));

    pi = PI->pi;

    if(pi->name){
        free((char *)pi->name);
        pi->name = NULL;
    }
    if(pi->data){
        free((char *)pi->data);
        pi->data = NULL;
    }

    return PI;
}/*end clear_xpin_pi()*/    

    

struct xpin_pi *
set_xpin_pi
(
    struct xpin_pi  *PI,
    const char      *name,
    const char      *data
)
{
    struct xpin_pi_  *pi = NULL;

    assert((PI != NULL) && (PI->pi != NULL));

    pi = PI->pi;

    pi->name = name;
    pi->data = data;

    return PI;
}/*end set_xpin_pi()*/ 



/* PI DESTRUCTOR: */
void
free_xpin_pi(struct xpin_pi *PI)
{
    struct xpin_pi_  *pi = NULL;

    if(PI == NULL)
        return;

    
    if(pi){
        if(pi->name) free((char *)pi->name);
        if(pi->data) free((char *)pi->data);
        free(pi);
    }
    free(PI);

    return;
}/*end free_xpin_pi()*/



/* that's all, folks! */
