/* xpinman.c
** XpinDoc "xpinman"
** wcm, 2003.02.04 - 2003.10.01
** ===
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>

#include "xpindoc.h"
#include "xpin_macros.h"

#define SBUF_SIZE  512


void
my_set_handler(XpinDoc X, const char *pattern, Xpin_Handler callback);


/* error trapping wrapper for set_handler(): */
void
my_set_handler(XpinDoc X, const char *pattern, Xpin_Handler callback)
{
    int   err = 0;

    err = X->set_handler(X, pattern, callback);
    if(err){
        fprintf(stderr, "xpinman error in: my_set_handler(X, %s, callback)\n",
                pattern);
        exit(EXIT_FAILURE);
    }

    return;
}/*end my_set_handler()*/


void
html_filter(XpinDoc X, const char *data, int len)
{
    char       *c = (char *)data;

    while(len){
        if(*c == '<') X->put_string(X, "&lt;");
        else if(*c == '>') X->put_string(X, "&gt;");
        else if(*c == '&') X->put_string(X, "&amp;");
        else X->put_char(X, *c);
        ++c;
        --len;
    }

    return;
}

void
xpinman_css(XpinDoc xpin)
{
    Xpin_PI   pi = xpin->pi(xpin);

    /* a pi handler for target "html-css" */

    xpin->printf(xpin, "<!--\n  %s\n-->\n", "xpinman_css() handler");
    xpin->printf(xpin, "<!--\n  target = [%s], data = [%s]\n-->\n",
        pi->target(pi), pi->data(pi) );

    return;
}


void
xpinman_start_document(XpinDoc xpin)
{
    char  xhtml_prologue[] =
        "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" "
        "\"xhtml-strict.dtd\">\n";

    xpin->put_string(xpin, xhtml_prologue);
    xpin->put_string(xpin, "<html>\n");
    return;
}


void
xpinman_end_document(XpinDoc xpin)
{
    time_t  now = time(NULL);

    xpin->printf(xpin, "\n<!--\n  %s, %s-->\n",
        "processed by xpinman", ctime(&now));

    xpin->put_string(xpin, "</html>\n");
    return;
}

/* element handlers for ignore: */
/*
   note:
   xpin->ignore fails for nested ignored elements
*/
void
xpinman_start_ignore(XpinDoc xpin)
{
    xpin->ignore_on(xpin);
}

void
xpinman_end_ignore(XpinDoc xpin)
{
    xpin->ignore_off(xpin);
} 


void
xpinman_pi(XpinDoc xpin)
{
    Xpin_PI   pi = xpin->pi(xpin);
    char      sbuf[SBUF_SIZE];


    sprintf(sbuf, "<!-- pi name:[%s], data:[%s] -->\n",
           pi->target(pi), pi->data(pi));
    xpin->put_string(xpin, sbuf);
}


void
xpinman_sdata(XpinDoc xpin)
{
    Xpin_Node            node = xpin->node(xpin);
    const char          *sdata = NULL;
    const char          *text  = NULL;

    sdata = node->attr_value(node, "sdata");
    text = node->attr_value(node, "text");

    if(!sdata) return;

    if(strcmp(sdata, "[GROFF]") == 0)
        xpin->put_string(xpin, "<tt>groff</tt>");
    else if(strcmp(sdata, "[LATEX]") == 0)
        xpin->put_string(xpin,
            "L<small><sup>A</sup></small>T<small><sub>E</sub></small>X");
    else if(text)
        xpin->put_string(xpin, text);

    return;
}



void
xpinman_head(XpinDoc xpin)
{
    char  meta[]=
        "<meta name=\"generator\" content=\"XpinDoc\" />\n";
    char  link_css[] =
        "<link rel=\"stylesheet\" type=\"text/css\"\n"
        "  href=\"xpinman.css\"  title=\"xpinman basic\" />\n";

    /* note use of xpin_macros.h: */

    IF_START(xpin){
        //xpin->put_string(xpin, "<head>\n");
        PUT_STRING(xpin, "<head>\n");
        PUT_STRING(xpin, meta);
        PUT_STRING(xpin, link_css);
    }
    else IF_END(xpin){
        PUT_STRING(xpin, "</head>\n");
    }

    return;
}



void
xpinman_title(XpinDoc xpin)
{
    Xpin_PI              pi = PI(xpin);
    char                 sbuf[SBUF_SIZE];

    /* note usage of xpin_macros.h: */
    IF_START(xpin){
        snprintf(sbuf, sizeof(sbuf),
                 "<!-- we are dealing with PI name:[%s] data:[%s] -->\n",
                 //pi->name(pi), pi->data(pi));
                 PI_TARGET(pi), PI_DATA(pi) );
        xpin->put_string(xpin, sbuf);
        xpin->put_string(xpin, "<title>");
    }
    IF_END(xpin){
        xpin->put_string(xpin, "</title>");
    }

    return;
}


void
xpinman_html_comment(XpinDoc xpin)
{
    IF_START(xpin){
        xpin->put_string(xpin, "\n<!--\n");
    }
    IF_END(xpin){
        xpin->put_string(xpin, "\n-->\n");
    }

    return;
}


void
xpinman_body(XpinDoc xpin)
{
    int   event_type = xpin->event_type(xpin);

    if(event_type == XPIN_START_ELEMENT)
        xpin->put_string(xpin, "<body>\n");
    else if(event_type == XPIN_END_ELEMENT)
        xpin->put_string(xpin, "</body>\n");
    else
        return;
}


void
xpinman_section(XpinDoc xpin)
{
    Xpin_Node            node = xpin->node(xpin);
    const char          *section_name = NULL;
    char                 sbuf[SBUF_SIZE];

    section_name = node->attr_value(node, "name");
    if(section_name){
        xpin->put_string(xpin, "<!--\n");
        sprintf(sbuf, "# === section %s ===\n", section_name);
        xpin->put_string(xpin, sbuf);
        xpin->put_string(xpin, "-->\n");
    }
    return;
}


void
xpinman_section_heading(XpinDoc xpin)
{
    Xpin_Node            node = xpin->node(xpin);
    Xpin_Node            parent = node->parent(node);
    const char          *section_name = NULL;

    if(xpin->event_type(xpin) == XPIN_START_ELEMENT){
        section_name = parent->attr_value(parent, "name");
        if(section_name && (strcmp(section_name, "title") == 0)){
            xpin->put_string(xpin, "<h1>");
        }
        else{
            xpin->put_string(xpin, "<h2 class=\"section\">");
        }
    }
    else if(xpin->event_type(xpin) == XPIN_END_ELEMENT){
        section_name = parent->attr_value(parent, "name");
        if(section_name && (strcmp(section_name, "title") == 0)){
            xpin->put_string(xpin, "</h1>\n");
        }
        else{
            xpin->put_string(xpin, "</h2>\n");
        }
    }
    return;
}


void
xpinman_para(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "\n<p>");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</p>\n");
    }
    return;
}


void
xpinman_codeblock(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin);
    Xpin_Text  T = xpin->node_text(xpin);

    /*
        yes, output mode, but notice how we can use the
        Xpin_Text object available in the current node to
        capture content with the output stack mechanism
    */

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<div class=\"codeblock\">\n");
        xpin->put_string(xpin, "<pre>\n");
        xpin->push_output(xpin, T, XPINOUT_TEXT);
    }
    else if(event_type == XPIN_END_ELEMENT){
        T = (Xpin_Text)xpin->pop_output(xpin);
        /* tighten up leading/trailing space: */
        T->trim(T);
        xpin->put_string(xpin, T->content(T));
        xpin->put_string(xpin, "\n</pre></div>\n");
    }
    return;
}


void
xpinman_commandline(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<div class=\"commandline\">\n");
        xpin->put_string(xpin, "<tt>\n");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</tt>\n</div>\n");
    }
    return;
}


void
xpinman_inline_bolder(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<b>");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</b>");
    }
    return;
}


void
xpinman_inline_ttype(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<tt>");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</tt>");
    }
    return;
}


void
xpinman_inline_ital(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<i>");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</i>");
    }
    return;
}


void
xpinman_list(XpinDoc xpin)
{
    int                  event_type = xpin->event_type(xpin); 
    Xpin_Node            node = xpin->node(xpin);
    const char          *list_class = NULL;

    list_class = node->attr_value(node, "class");

    if(event_type == XPIN_START_ELEMENT){
        if(strcmp(list_class, "unordered") == 0){
            xpin->put_string(xpin, "<ul>\n");
        }
        else if(strcmp(list_class, "ordered") == 0){
            xpin->put_string(xpin, "<ol>\n");
        }
    }
    else if(event_type == XPIN_END_ELEMENT){
        if(strcmp(list_class, "unordered") == 0){
            xpin->put_string(xpin, "</ul>\n");
        }
        else if(strcmp(list_class, "ordered") == 0){
            xpin->put_string(xpin, "</ol>\n");
        }
    }
    return;
}



void
xpinman_item(XpinDoc xpin)
{
    int        event_type = xpin->event_type(xpin); 

    if(event_type == XPIN_START_ELEMENT){
        xpin->put_string(xpin, "<li>");
    }
    else if(event_type == XPIN_END_ELEMENT){
        xpin->put_string(xpin, "</li>");
    }
    return;
}

void
xpinman_deflist(XpinDoc xpin)
{

    IF_START(xpin) {
        xpin->put_string(xpin, "<dl>\n");
    }
    else IF_END(xpin) {
        xpin->put_string(xpin, "</dl>\n");
    }

    return;
}


void
xpinman_defterm(XpinDoc xpin)
{

    IF_START(xpin) {
        xpin->put_string(xpin, "<dt>");
    }
    else IF_END(xpin) {
        xpin->put_string(xpin, "</dt>");
    }

    return;
}


void
xpinman_defitem(XpinDoc xpin)
{

    IF_START(xpin) {
        xpin->put_string(xpin, "<dd>");
    }
    else IF_END(xpin) {
        xpin->put_string(xpin, "</dd>");
    }
}


int
xpindoc_main(int argc, char **argv)
{
    FILE    *input = NULL;
    FILE    *tidy = NULL;
    XpinDoc  xpin = NULL;
    int      err = 0;
    char    *mydata = NULL;

    input = stdin;

#if 0
    tidy = popen("tidy -q -asxml", "w");
    if(!tidy){
        fprintf(stderr, "error opening pipe\n");
        exit(EXIT_FAILURE);
    }
    pclose(tidy);
#endif

    xpin = new_xpindoc(XPIN_DEFAULT);
    if(xpin == NULL){
        fprintf(stderr, "error creating xpindoc\n");
        exit(EXIT_FAILURE);
    }

    mydata = strdup("This is mydata!");

    xpin->set_mydata(xpin, (void *)mydata);

    xpin->set_datamode(xpin, XPIN_OUTPUT);
    xpin->set_filter(xpin, &html_filter);
    //xpin->push_output(xpin, tidy, XPINOUT_PIPE); 


    my_set_handler(xpin, "start_document", &xpinman_start_document);
    my_set_handler(xpin, "end_document", &xpinman_end_document);
    my_set_handler(xpin, "<?html-css?>", &xpinman_css);
    my_set_handler(xpin, "</sdata>", &xpinman_sdata);
    my_set_handler(xpin, "<head/>", &xpinman_head);
    my_set_handler(xpin, "<title/>", &xpinman_title);
    my_set_handler(xpin, "<description/>", &xpinman_html_comment);
    my_set_handler(xpin, "<author/>", &xpinman_html_comment);
    my_set_handler(xpin, "<body/>", &xpinman_body);
    my_set_handler(xpin, "<section>", &xpinman_section);
    my_set_handler(xpin, "/*/section/heading", &xpinman_section_heading);
    my_set_handler(xpin, "<para/>", &xpinman_para);
    my_set_handler(xpin, "<codeblock/>", &xpinman_codeblock);
    my_set_handler(xpin, "<commandline/>", &xpinman_commandline);
    my_set_handler(xpin, "/*/para/emph", &xpinman_inline_bolder);
    my_set_handler(xpin, "/*/emph", &xpinman_inline_bolder);
    my_set_handler(xpin, "<progname/>", &xpinman_inline_bolder);
    my_set_handler(xpin, "<filename/>", &xpinman_inline_ttype);
    my_set_handler(xpin, "<function/>", &xpinman_inline_ttype);
    my_set_handler(xpin, "/*/function/arg", &xpinman_inline_ital);
    my_set_handler(xpin, "<object/>", &xpinman_inline_ttype);
    my_set_handler(xpin, "<list/>", &xpinman_list);
    my_set_handler(xpin, "<item/>", &xpinman_item);
    my_set_handler(xpin, "/*/item/list", &xpinman_list);
    my_set_handler(xpin, "<deflist/>", &xpinman_deflist);
    my_set_handler(xpin, "<defterm/>", &xpinman_defterm);
    my_set_handler(xpin, "/*/deflist/item", &xpinman_defitem);

    err = xpin->parse_stream(xpin, input);
    if(err){
        fprintf(stderr, "error during parse\n");
        exit(EXIT_FAILURE);
    }

    free(mydata);
    xpin->free(xpin);

    return 0;
}/*end xpinman_main()*/


int
main(int argc, char **argv)
{
    return xpindoc_main(argc, argv);
}

/* that's all, folks! */
