/* xpintags.c
** XpinDoc "xpintags"
**
** print tags from an xml file with xpindoc in output mode
** test namespace support
**
** wcm, 2003.09.23 - 2003.09.29
** ===
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xpindoc.h"
#include "xpin_macros.h"

#define  SBUF_SIZE  512

/* xpintags prototypes: */
void
my_set_handler(XpinDoc X, const char *pattern, Xpin_Handler callback);


void
xpincap_start_document(XpinDoc xpin);

void
xpincap_end_document(XpinDoc xpin);

void
xpincap_start_element(XpinDoc xpin);

void
xpintags_ns_element(XpinDoc xpin);

/* definitions: */

/* error trapping wrapper for set_handler(): */
void
my_set_handler(XpinDoc X, const char *pattern, Xpin_Handler callback)
{
    int   err = 0;

    err = X->set_handler(X, pattern, callback);
    if(err){
        fprintf(stderr, "xpinman error in: "
                        "my_set_handler(X, %s, callback)\n", pattern);
        exit(EXIT_FAILURE);
    }

    return;
}/*end my_set_handler()*/


void
xpintags_start_document(XpinDoc xpin)
{
    xpin->put_string(xpin, (char *)xpin->mydata(xpin));
    xpin->put_string(xpin, "\n");

    return;
}


void
xpintags_end_document(XpinDoc xpin)
{
    xpin->put_string(xpin, "Done.\n");

    return;
}

void
xpintags_ns_element(XpinDoc xpin)
{
    Xpin_Node        N = xpin->node(xpin);
    char             sbuf[SBUF_SIZE];
    int              i = 0;

    if(xpin->event_type(xpin) == XPIN_START_ELEMENT){
        /* skip indent for depth = 0: */ 
        for(i = 1; i < N->depth(N); ++i){
            xpin->put_string(xpin, "    ");
        }

        snprintf(sbuf, sizeof(sbuf), "%s{ns_uri:%s}",
            N->qname(N), N->ns_uri(N) );
        xpin->put_string(xpin, sbuf);

        if(N->attr_count(N) > 0){
            struct xpin_attr  **attributes = N->attributes(N);

            for(i = 0; i < N->attr_count(N); ++i){
                struct xpin_attr  *attr = attributes[i];
                char               sbuf[SBUF_SIZE];

                if(attr->ns_uri){
                    snprintf(sbuf, sizeof(sbuf), " [%s{%s}='%s']",
                        attr->name, attr->ns_uri, attr->value);
                    xpin->put_string(xpin, sbuf);
                }
                else{
                    snprintf(sbuf, sizeof(sbuf), " [%s='%s']",
                        attr->name, attr->value);
                    xpin->put_string(xpin, sbuf);
                }
            }
        }

        xpin->put_char(xpin, '\n');
    }

    else{
        /* end element: do nothing */
    }

    return;
}
        


void
xpintags_start_element(XpinDoc xpin)
{
    Xpin_Node        N = xpin->node(xpin);
    int              i = 0;

    /* skip indent for depth = 0: */ 
    for(i = 1; i < N->depth(N); ++i){
        xpin->put_string(xpin, "    ");
    } 
    xpin->put_string(xpin, xpin->tagname(xpin));
    if(N->ns_uri(N)){
        char   sbuf[SBUF_SIZE];

        snprintf(sbuf, sizeof(sbuf), " {%s}", N->ns_uri(N) );
        xpin->put_string(xpin, sbuf);
    }

    if(N->attr_count(N) > 0){
        struct xpin_attr  **attributes = N->attributes(N);
        for(i = 0; i < N->attr_count(N); ++i){
            struct xpin_attr  *attr = attributes[i];
            char               sbuf[SBUF_SIZE];

            if(attr->ns_uri){
                snprintf(sbuf, sizeof(sbuf), " [%s{%s}='%s']",
                    attr->name, attr->ns_uri, attr->value);
                xpin->put_string(xpin, sbuf);
            }
            else{
                snprintf(sbuf, sizeof(sbuf), " [%s='%s']",
                    attr->name, attr->value);
                xpin->put_string(xpin, sbuf);

            }
        }
    }

    xpin->put_char(xpin, '\n');

    return;
}



int
xpindoc_main(int argc, char **argv)
{
    FILE    *input = NULL;
    XpinDoc  xpin = NULL;
    int      err = 0;
    char    *mydata = NULL;

    input = stdin;

    /* process namespaces: */
    xpin = new_xpindoc(XPIN_NAMESPACE | XPIN_QNAME);
    if(xpin == NULL){
        fprintf(stderr, "error creating xpindoc\n");
        exit(EXIT_FAILURE);
    }

    mydata = strdup("This is xpintags!");

    xpin->set_mydata(xpin, (void *)mydata);

    /* use xpin_writer for output: */
    xpin->set_datamode(xpin, XPIN_OUTPUT);
    /* ignore content: */
    xpin->ignore_on(xpin);

    /* our handlers: */
    my_set_handler(xpin, "Start_Document", &xpintags_start_document);
    my_set_handler(xpin, "NS_Start_Element", &xpintags_ns_element);
    my_set_handler(xpin, "Start_Element", &xpintags_start_element);
    my_set_handler(xpin, "End_Document", &xpintags_end_document);

    err = xpin->parse_stream(xpin, input);
    if(err){
        fprintf(stderr, "error during parse\n");
        exit(EXIT_FAILURE);
    }

    free(mydata);
    xpin->free(xpin);

    return 0;
}/*end xpindoc_main()*/


int
main(int argc, char **argv)
{
    return xpindoc_main(argc, argv);
}

/* that's all, folks! */
