/* xpintest.c
** testing XpinDoc
** wcm, 2002.11.22 - 2003.02.04
** ===
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xpindoc.h"

void
my_start_document(XpinDoc xpin)
{
    printf("Welcome to my Start Document Handler!\n");
}


void
my_end_document(XpinDoc xpin)
{
    printf("Farewell to this document!\n");
}


void
my_start_element(XpinDoc xpin)
{
    Xpin_Node            node = xpin->node(xpin);
    int                  attr_count = node->attr_count(node);
    Xpin_Attributes      attrs = node->attributes(node);
    int                  i = 0;
    char                *myvalue = NULL;

    printf("Arrived in my Start Element Handler!\n");
    printf("  the node depth is: %d\n", xpin->depth(xpin));
    printf("  the tagname is: %s\n", xpin->tagname(xpin));
    printf("  the tagpath is: %s\n", xpin->tagpath(xpin));
    printf("  attr_count is: %d\n", attr_count);
    for(i = 0; i < attr_count; ++i){
        printf("  * attribute[%d]->name: %s\n", i, attrs[i]->name);
        printf("  * attribute[%d]->value: %s\n", i, attrs[i]->value);
    }

    myvalue = node->attr_value(node, "title");
    if(myvalue)
        printf("  attribute lookup found value: %s\n", myvalue);

}


void
my_end_element(XpinDoc xpin)
{
    Xpin_Node  node = xpin->node(xpin);

    printf("Arrived in my End Element Handler!\n");
    printf("  the node depth is: %d\n", xpin->depth(xpin));
    printf("  the tagname is: %s\n", xpin->tagname(xpin));
    printf("  the tagpath is: %s\n", xpin->tagpath(xpin));
    printf("  the content was: %s\n", 
        xpin->content(xpin) ? xpin->content(xpin) : "[empty]" );

    node->content_normalize(node);
    node->content_upper(node);
    printf("  the content now: %s\n", 
        xpin->content(xpin) ? xpin->content(xpin) : "[empty]" );

    

    return;
} 


void
my_pi(XpinDoc xpin)
{
    Xpin_Event event = xpin->event(xpin);
    Xpin_PI    pi = event->pi(event);
    Xpin_PI    pi2 = (Xpin_PI)event->data(event);

    printf("Arrived in my PI Handler!\n");
    printf("  pi_name is %s\n", pi->name(pi));
    printf("  pi_data is %s\n", pi->data(pi));
    printf("  pi2_name is %s\n", pi2->name(pi));
    printf("  pi2_data is %s\n", pi2->data(pi));

    return;
}


void
my_from(XpinDoc xpin)
{
    printf("*** The <from> Handler ***\n");
    printf("***   See Ya Later     ***\n");
    return;
}

void
my_from_end(XpinDoc xpin)
{
    printf("*** The </from> Handler ***\n");
    printf("===> the content is: %s\n",
        xpin->content(xpin) ? xpin->content(xpin) : "[empty]");
    printf("***   See Ya Later     ***\n");
    return;
}

void
my_emph(XpinDoc xpin)
{
    Xpin_Event event = xpin->event(xpin);
    Xpin_Node  node = xpin->node(xpin);

    printf(";;;;; the emph handler ;;;;;\n");
    if(event->type(event) == XPIN_START_ELEMENT){
        printf("!!! BEGIN EMPH !!!\n");
    }
    else if(event->type(event) == XPIN_END_ELEMENT){
        node->content_prepend(node, "-->");
        node->content_append(node, "<--");
        printf("shazam: [%s]\n",
          xpin->content(xpin) ? xpin->content(xpin) : "[empty]");
        printf("!!! DONE WITH EMPH !!!\n");
        node->content_merge(node);
    }

    return;
}


int
maintest(void)
{
    FILE    *input = NULL;
    XpinDoc  xpin = NULL;
    int      err = 0;
    char    *mydata = NULL;

    input = stdin;
    xpin = new_xpindoc();
    if(xpin == NULL){
        fprintf(stderr, "error creating xpindoc\n");
        exit(EXIT_FAILURE);
    }

    mydata = strdup("This is mydata!");

    printf("before setting mydata\n");
    xpin->set_mydata(xpin, (void *)mydata);
    printf("after setting mydata\n");

    xpin->set_handler(xpin, "start_document", &my_start_document);
    xpin->set_handler(xpin, "start_element", &my_start_element);
    xpin->set_handler(xpin, "end_element", &my_end_element);
    xpin->set_handler(xpin, "end_document", &my_end_document);
    xpin->set_handler(xpin, "processing_instruction", &my_pi);
    xpin->set_handler(xpin, "<from>", &my_from);
    xpin->set_handler(xpin, "</from>", &my_from_end);
    xpin->set_handler(xpin, "S:/*/para/emph", &my_emph);
    xpin->set_handler(xpin, "E:/*/para/emph", &my_emph);

    printf("before starting parse\n");
    err = xpin->parse_stream(xpin, input);
    printf("after starting parse\n");
    if(err){
        fprintf(stderr, "error during parse\n");
        exit(EXIT_FAILURE);
    }

    free(mydata);
    xpin->free(xpin);

    return 0;
}/*end maintest()*/

int
main(int argc, char **argv)
{
    printf("beginning XpinDoc...\n");
    maintest();
    printf("XpinDoc done!\n");
    return 0;
}

/* that's all, folks! */

